@@ -3,7 +3,7 @@
 
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
 # *                                                                         *
-# * The Nmap Security Scanner is (C) 1996-2011 Insecure.Com LLC. Nmap is    *
+# * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
 # * also a registered trademark of Insecure.Com LLC.  This program is free  *
 # * software; you may redistribute and/or modify it under the terms of the  *
 # * GNU General Public License as published by the Free Software            *
@@ -13,11 +13,12 @@
 # * technology into proprietary software, we sell alternative licenses      *
 # * (contact sales@insecure.com).  Dozens of software vendors already       *
 # * license Nmap technology such as host discovery, port scanning, OS       *
-# * detection, and version detection.                                       *
+# * detection, version detection, and the Nmap Scripting Engine.            *
 # *                                                                         *
 # * Note that the GPL places important restrictions on "derived works", yet *
 # * it does not provide a detailed definition of that term.  To avoid       *
-# * misunderstandings, we consider an application to constitute a           *
+# * misunderstandings, we interpret that term as broadly as copyright law   *
+# * allows.  For example, we consider an application to constitute a        *
 # * "derivative work" for the purpose of this license if it does any of the *
 # * following:                                                              *
 # * o Integrates source code from Nmap                                      *
@@ -31,19 +32,20 @@
 # * o Links to a library or executes a program that does any of the above   *
 # *                                                                         *
 # * The term "Nmap" should be taken to also include any portions or derived *
-# * works of Nmap.  This list is not exclusive, but is meant to clarify our *
-# * interpretation of derived works with some common examples.  Our         *
-# * interpretation applies only to Nmap--we don't speak for other people's  *
-# * GPL works.                                                              *
+# * works of Nmap, as well as other software we distribute under this       *
+# * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
+# * but is meant to clarify our interpretation of derived works with some   *
+# * common examples.  Our interpretation applies only to Nmap--we don't     *
+# * speak for other people's GPL works.                                     *
 # *                                                                         *
 # * If you have any questions about the GPL licensing restrictions on using *
 # * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
 # * we also offer alternative license to integrate Nmap into proprietary    *
 # * applications and appliances.  These contracts have been sold to dozens  *
 # * of software vendors, and generally include a perpetual license as well  *
-# * as providing for priority support and updates as well as helping to     *
-# * fund the continued development of Nmap technology.  Please email        *
-# * sales@insecure.com for further information.                             *
+# * as providing for priority support and updates.  They also fund the      *
+# * continued development of Nmap.  Please email sales@insecure.com for     *
+# * further information.                                                    *
 # *                                                                         *
 # * As a special exception to the GPL terms, Insecure.Com LLC grants        *
 # * permission to link the code of this program with any version of the     *
@@ -67,15 +69,16 @@
 # * and add new features.  You are highly encouraged to send your changes   *
 # * to nmap-dev@insecure.org for possible incorporation into the main       *
 # * distribution.  By sending these changes to Fyodor or one of the         *
-# * Insecure.Org development mailing lists, it is assumed that you are      *
-# * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
-# * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
-# * will always be available Open Source, but this is important because the *
-# * inability to relicense code has caused devastating problems for other   *
-# * Free Software projects (such as KDE and NASM).  We also occasionally    *
-# * relicense the code to third parties as discussed above.  If you wish to *
-# * specify special license conditions of your contributions, just say so   *
-# * when you send them.                                                     *
+# * Insecure.Org development mailing lists, or checking them into the Nmap  *
+# * source code repository, it is understood (unless you specify otherwise) *
+# * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+# * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+# * code.  Nmap will always be available Open Source, but this is important *
+# * because the inability to relicense code has caused devastating problems *
+# * for other Free Software projects (such as KDE and NASM).  We also       *
+# * occasionally relicense the code to third parties as discussed above.    *
+# * If you wish to specify special license conditions of your               *
+# * contributions, just say so when you send them.                          *
 # *                                                                         *
 # * This program is distributed in the hope that it will be useful, but     *
 # * WITHOUT ANY WARRANTY; without even the implied warranty of              *
@@ -111,7 +114,6 @@ class HostDetails(HIGVBox):
 
         self.__create_widgets()
 
-        self.set_comment(host.comment)
         self.set_os_image(get_os_logo(host))
 
         self.set_vulnerability_image(get_vulnerability_logo(host.get_open_ports()))
@@ -138,12 +140,12 @@ class HostDetails(HIGVBox):
         os = host.get_best_osmatch()
         if os:
             os['portsused'] = host.get_ports_used()
-            os['osclass'] = host.get_osclasses()
 
         self.set_os(os)
         self.set_tcpseq(host.get_tcpsequence())
         self.set_ipseq(host.get_ipidsequence())
         self.set_tcptsseq(host.get_tcptssequence())
+        self.set_comment(host.comment)
 
     def __create_widgets(self):
         self.host_status_expander = gtk.Expander('<b>'+_('Host Status')+'</b>')
@@ -151,7 +153,7 @@ class HostDetails(HIGVBox):
         self.hostnames_expander = gtk.Expander('<b>'+_('Hostnames')+'</b>')
         self.os_expander = gtk.Expander('<b>'+_('Operating System')+'</b>')
         self.portsused_expander = gtk.Expander('<b>'+_('Ports used')+'</b>')
-        self.osclass_expander = gtk.Expander('<b>'+_('OS Class')+'</b>')
+        self.osclass_expander = gtk.Expander('<b>'+_('OS Classes')+'</b>')
         self.tcp_expander = gtk.Expander('<b>'+_('TCP Sequence')+'</b>')
         self.ip_expander = gtk.Expander('<b>'+_('IP ID Sequence')+'</b>')
         self.tcpts_expander = gtk.Expander('<b>'+_('TCP TS Sequence')+'</b>')
@@ -365,7 +367,7 @@ class HostDetails(HIGVBox):
             except:pass
 
             try:
-                self.set_osclass(os['osclass'])
+                self.set_osclass(os['osclasses'])
                 self.osclass_expander.set_use_markup(True)
                 table.attach(self.osclass_expander,0,2,y1,y2)
             except:pass
