@@ -3,7 +3,7 @@
 
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
 # *                                                                         *
-# * The Nmap Security Scanner is (C) 1996-2011 Insecure.Com LLC. Nmap is    *
+# * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
 # * also a registered trademark of Insecure.Com LLC.  This program is free  *
 # * software; you may redistribute and/or modify it under the terms of the  *
 # * GNU General Public License as published by the Free Software            *
@@ -13,11 +13,12 @@
 # * technology into proprietary software, we sell alternative licenses      *
 # * (contact sales@insecure.com).  Dozens of software vendors already       *
 # * license Nmap technology such as host discovery, port scanning, OS       *
-# * detection, and version detection.                                       *
+# * detection, version detection, and the Nmap Scripting Engine.            *
 # *                                                                         *
 # * Note that the GPL places important restrictions on "derived works", yet *
 # * it does not provide a detailed definition of that term.  To avoid       *
-# * misunderstandings, we consider an application to constitute a           *
+# * misunderstandings, we interpret that term as broadly as copyright law   *
+# * allows.  For example, we consider an application to constitute a        *
 # * "derivative work" for the purpose of this license if it does any of the *
 # * following:                                                              *
 # * o Integrates source code from Nmap                                      *
@@ -31,19 +32,20 @@
 # * o Links to a library or executes a program that does any of the above   *
 # *                                                                         *
 # * The term "Nmap" should be taken to also include any portions or derived *
-# * works of Nmap.  This list is not exclusive, but is meant to clarify our *
-# * interpretation of derived works with some common examples.  Our         *
-# * interpretation applies only to Nmap--we don't speak for other people's  *
-# * GPL works.                                                              *
+# * works of Nmap, as well as other software we distribute under this       *
+# * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
+# * but is meant to clarify our interpretation of derived works with some   *
+# * common examples.  Our interpretation applies only to Nmap--we don't     *
+# * speak for other people's GPL works.                                     *
 # *                                                                         *
 # * If you have any questions about the GPL licensing restrictions on using *
 # * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
 # * we also offer alternative license to integrate Nmap into proprietary    *
 # * applications and appliances.  These contracts have been sold to dozens  *
 # * of software vendors, and generally include a perpetual license as well  *
-# * as providing for priority support and updates as well as helping to     *
-# * fund the continued development of Nmap technology.  Please email        *
-# * sales@insecure.com for further information.                             *
+# * as providing for priority support and updates.  They also fund the      *
+# * continued development of Nmap.  Please email sales@insecure.com for     *
+# * further information.                                                    *
 # *                                                                         *
 # * As a special exception to the GPL terms, Insecure.Com LLC grants        *
 # * permission to link the code of this program with any version of the     *
@@ -67,15 +69,16 @@
 # * and add new features.  You are highly encouraged to send your changes   *
 # * to nmap-dev@insecure.org for possible incorporation into the main       *
 # * distribution.  By sending these changes to Fyodor or one of the         *
-# * Insecure.Org development mailing lists, it is assumed that you are      *
-# * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
-# * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
-# * will always be available Open Source, but this is important because the *
-# * inability to relicense code has caused devastating problems for other   *
-# * Free Software projects (such as KDE and NASM).  We also occasionally    *
-# * relicense the code to third parties as discussed above.  If you wish to *
-# * specify special license conditions of your contributions, just say so   *
-# * when you send them.                                                     *
+# * Insecure.Org development mailing lists, or checking them into the Nmap  *
+# * source code repository, it is understood (unless you specify otherwise) *
+# * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+# * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+# * code.  Nmap will always be available Open Source, but this is important *
+# * because the inability to relicense code has caused devastating problems *
+# * for other Free Software projects (such as KDE and NASM).  We also       *
+# * occasionally relicense the code to third parties as discussed above.    *
+# * If you wish to specify special license conditions of your               *
+# * contributions, just say so when you send them.                          *
 # *                                                                         *
 # * This program is distributed in the hope that it will be useful, but     *
 # * WITHOUT ANY WARRANTY; without even the implied warranty of              *
@@ -342,10 +345,6 @@ class HostOpenPorts(HIGVBox):
         self.host_tree.set_default_sort_func(lambda *args: -1)
         self.port_list.set_default_sort_func(lambda *args: -1)
         self.port_tree.set_default_sort_func(lambda *args: -1)
-        self.host_list.set_sort_column_id(-1, gtk.SORT_ASCENDING)
-        self.host_tree.set_sort_column_id(-1, gtk.SORT_ASCENDING)
-        self.port_list.set_sort_column_id(-1, gtk.SORT_ASCENDING)
-        self.port_tree.set_sort_column_id(-1, gtk.SORT_ASCENDING)
         self.frozen_host_view_model = self.host_view.get_model()
         self.frozen_port_view_model = self.port_view.get_model()
         self.host_view.freeze_child_notify()
@@ -356,10 +355,14 @@ class HostOpenPorts(HIGVBox):
     def thaw(self):
         """Restore notifications and sorting (after making changes to the
         model)."""
-        self.host_list.set_sort_column_id(*self.frozen_host_list_sort_column_id)
-        self.host_tree.set_sort_column_id(*self.frozen_host_tree_sort_column_id)
-        self.port_list.set_sort_column_id(*self.frozen_port_list_sort_column_id)
-        self.port_tree.set_sort_column_id(*self.frozen_port_tree_sort_column_id)
+        if self.frozen_host_list_sort_column_id != (None, None):
+            self.host_list.set_sort_column_id(*self.frozen_host_list_sort_column_id)
+        if self.frozen_host_tree_sort_column_id != (None, None):
+            self.host_tree.set_sort_column_id(*self.frozen_host_tree_sort_column_id)
+        if self.frozen_port_list_sort_column_id != (None, None):
+            self.port_list.set_sort_column_id(*self.frozen_port_list_sort_column_id)
+        if self.frozen_port_tree_sort_column_id != (None, None):
+            self.port_tree.set_sort_column_id(*self.frozen_port_tree_sort_column_id)
         self.host_view.set_model(self.frozen_host_view_model)
         self.port_view.set_model(self.frozen_port_view_model)
         self.host_view.thaw_child_notify()
